package de.intevation.flys.artifacts.states;

import org.apache.log4j.Logger;

import org.w3c.dom.Element;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;
import de.intevation.artifacts.CallMeta;

import de.intevation.artifacts.common.utils.XMLUtils;

import de.intevation.artifactdatabase.ProtocolUtils;
import de.intevation.artifactdatabase.data.StateData;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.resources.Resources;

/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class CalculationSelect extends DefaultState {

    /** The logger that is used in this class.*/
    private static Logger logger = Logger.getLogger(CalculationSelect.class);



    public static final String FIELD_MODE = "calculation_mode";

    /** Constant value for the reference line calculation.*/
    public static final String CALCULATION_SURFACE_CURVE =
        "calc.surface.curve";

    /** Constant value for the differences calculation.*/
    public static final String CALCULATION_DURATION_CURVE =
        "calc.duration.curve";

    /** Constant value for the flood map calculation.*/
    public static final String CALCULATION_FLOOD_MAP =
        "calc.flood.map";

    /** Constant value for the profile calculation.*/
    public static final String CALCULATION_DISCHARGE_LONGITUDINAL_CURVE =
        "calc.discharge.longitudinal.section";

    /** Constant value for the state discharge curve calculation.*/
    public static final String CALCULATION_DISCHARGE_CURVE =
        "calc.discharge.curve";

    /** An array that holds all available calculation modes.*/
    public static final String[] CALCULATIONS = {
        CALCULATION_SURFACE_CURVE,
        CALCULATION_FLOOD_MAP,
        CALCULATION_DISCHARGE_CURVE,
        CALCULATION_DURATION_CURVE,
        CALCULATION_DISCHARGE_LONGITUDINAL_CURVE };


    /** Error message that is thrown if no mode has been chosen.*/
    public static final String ERROR_NO_CALCULATION_MODE =
        "error_feed_no_calculation_mode";

    /** Error message that is thrown if an invalid calculation mode has been
     * chosen.*/
    public static  final String ERROR_INVALID_CALCULATION_MODE =
        "error_feed_invalid_calculation_mode";


    public CalculationSelect() {
    }


    @Override
    protected Element[] createItems(
        XMLUtils.ElementCreator cr,
        Artifact    artifact,
        String      name,
        CallContext context)
    {
        CallMeta meta   = context.getMeta();
        Element[] calcs = new Element[CALCULATIONS.length];

        int i = 0;

        for (String calc: CALCULATIONS) {
            calcs[i++] = createItem(
                cr, new String[] {
                    Resources.getMsg(meta, calc, calc),
                    calc
                });
        }

        return calcs;
    }


    protected Element createItem(XMLUtils.ElementCreator cr, Object obj) {
        Element item  = ProtocolUtils.createArtNode(cr, "item", null, null);
        Element label = ProtocolUtils.createArtNode(cr, "label", null, null);
        Element value = ProtocolUtils.createArtNode(cr, "value", null, null);

        String[] arr = (String[]) obj;

        label.setTextContent(arr[0]);
        value.setTextContent(arr[1]);

        item.appendChild(label);
        item.appendChild(value);

        return item;
    }


    @Override
    public boolean validate(Artifact artifact)
    throws IllegalArgumentException
    {
        logger.debug("CalculationSelect.validate");
        FLYSArtifact flys = (FLYSArtifact) artifact;

        StateData data = getData(flys, FIELD_MODE);
        String    calc = data != null ? (String) data.getValue() : null;

        if (calc == null) {
            throw new IllegalArgumentException(ERROR_NO_CALCULATION_MODE);
        }

        calc = calc.trim().toLowerCase();

        for (String mode: CALCULATIONS) {
            if (mode.equals(calc)) {
                return true;
            }
        }

        throw new IllegalArgumentException(ERROR_INVALID_CALCULATION_MODE);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
